package com.cloudcross.ssp.operator.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.cloudcross.ssp.base.dao.GenericIBatisDao;
import com.cloudcross.ssp.common.Config;
import com.cloudcross.ssp.common.utils.Assert;
import com.cloudcross.ssp.common.utils.MapBuilder;
import com.cloudcross.ssp.common.utils.Pager;
import com.cloudcross.ssp.common.utils.SqlHelper;
import com.cloudcross.ssp.operator.model.Zone;
import com.cloudcross.ssp.operator.service.IZoneService;
import com.cloudcross.ssp.web.widget.SelectorController.ISelector;
import com.cloudcross.ssp.web.widget.SelectorController.ISelectorProvider;

    @Service("operator.zoneService")
    public class ZoneService implements IZoneService,ISelectorProvider{
	
	@Autowired
	private GenericIBatisDao myBatisDao;
	@Autowired
	Config config;
	//文件系统可上传的文件后缀集合
	private static Set<String> noStuffixSet = new HashSet<String>();
		
	//初始化可上传文件后缀
	static {
		noStuffixSet.add("jpg");
		noStuffixSet.add("gif");
	}
	
	@Override
	public boolean add( Zone t) {
		Date date = new Date();
		t.setUpdated(date);
		t.setSupdated(date);
		myBatisDao.save("operator.zoneSqlMapper.addZone", t);		
		return true;
	}

	@Override
	public int countByParams(Map<String, Object> paramMap) {
		String searchValue = SqlHelper.doLike(paramMap.get("searchValue"));
		paramMap.put("key", paramMap.get("searchKey"));
		paramMap.put("value", searchValue);
		paramMap.put("width", SqlHelper.doLike(paramMap.get("width")));
		paramMap.put("height", SqlHelper.doLike(paramMap.get("height")));
		paramMap.put("device", SqlHelper.doLike(paramMap.get("device")));
		paramMap.put("position", SqlHelper.doLike(paramMap.get("position")));
		paramMap.put("status", SqlHelper.doLike(paramMap.get("status")));
		return myBatisDao.get("operator.zoneSqlMapper.countByParams", paramMap);
	}

	@Override
	public boolean edit(Zone zone) {
		Date date = new Date();
		zone.setUpdated(date);
		zone.setSupdated(date);
		myBatisDao.save("operator.zoneSqlMapper.editZone", zone);
		return true;
	}
	@Override
	public Zone findZone(Long zoneId) {
		return myBatisDao.get("operator.zoneSqlMapper.findZone", zoneId);	
	}
	
	@Override
	public Zone findById(Long id) {
//		System.out.println("bbb"+id);
		return myBatisDao.get("operator.zoneSqlMapper.selectById", id);
	}

	@Override
	public List<Zone> findByParams(Map<String, Object> paramMap, Pager pager) {
		String searchValue = SqlHelper.doLike(paramMap.get("searchValue"));
		paramMap.put("key", paramMap.get("searchKey"));
		paramMap.put("value", searchValue);
		paramMap.put("pager", pager);
		paramMap.put("width", SqlHelper.doLike(paramMap.get("width")));
		paramMap.put("height", SqlHelper.doLike(paramMap.get("height")));
		paramMap.put("device", SqlHelper.doLike(paramMap.get("device")));
		paramMap.put("position", SqlHelper.doLike(paramMap.get("position")));
		paramMap.put("status", SqlHelper.doLike(paramMap.get("status")));
		return myBatisDao.getList("operator.zoneSqlMapper.selectByParams", paramMap);
	}
	@Override
	public boolean updateStatus(List<Long> zoneIdList, int status) {
		myBatisDao.save("operator.zoneSqlMapper.updateZoneStatus",
				MapBuilder.create(HashMap.class)
					.add("zoneIdList", zoneIdList)
					.add("updated", new Date())
					.add("status", status).map());
		return true;
	}
	
	//导出excel
	@Override
	public List<Zone> findDeliverEffectAnalysis(Map<String, Object> paramMap) {
		
		return myBatisDao.getList("operator.zoneSqlMapper.findDeliverEffectAnalysis", paramMap);
	}
	
	
	
	@Override
	public long findByName(String name) {
		// TODO Auto-generated method stub
		return myBatisDao.get("operator.zoneSqlMapper.findByName", name);
	}

	@Override
	public String deleteById(long id) {
		// TODO Auto-generated method stub
		return myBatisDao.get("operator.zoneSqlMapper.deleteById", id);
	}

	@Override
	public List<Zone> findByDeviceAndPlaceId(Map<String, Object> paramMap) {
		return myBatisDao.getList("operator.zoneSqlMapper.findByDeviceAndPlaceId", paramMap);
	}
	
	@Override
	public List<String> findAdgroupNameByZoneIdAndSchedule(Map<String, Object> paramMap) {
		// TODO Auto-generated method stub
		return myBatisDao.getList("operator.zoneSqlMapper.findAdgroupNameByZoneIdAndSchedule", paramMap);
	}
	
	/**
	 * 首页查找设备 用到，勿删
	 */
	@Override
	public List<Zone> findByDevice(Integer device){
		// TODO Auto-generated method stub
		return myBatisDao.getList("operator.zoneSqlMapper.findByDevice", device);
	}


	@Override
	public Map<String, List<? extends ISelector>> provideData(String[] parentId) {
		Assert.isTrue(ArrayUtils.isNotEmpty(parentId));
		Integer operatorId = Integer.parseInt(parentId[0]);
		List<Zone> zoneList = findByOperatorId(operatorId);
		return MapBuilder.create(HashMap.class).add("zoneList", zoneList).map();
	}

	@Override
	public List<Zone> findByOperatorId(Integer operatorId) {
		// TODO Auto-generated method stub
		return myBatisDao.getList("operator.zoneSqlMapper.findByOperatorId", operatorId);
	}
	
	/*public @ResponseBody AjaxResult fileUpload(HttpServletRequest request,HttpServletResponse response) {
		Map<String,String>map = new HashMap<String,String>();
		AjaxResult 	result = new AjaxResult();
		
		try {
			// 创建一个通用的多部分解析器
			CommonsMultipartResolver multipartResolver = new CommonsMultipartResolver(request.getSession().getServletContext());
			// 判断 request 是否有文件上传,即多部分请求
			if (multipartResolver.isMultipart(request)) {
				// 转换成多部分request
				MultipartHttpServletRequest multiRequest = (MultipartHttpServletRequest) request;
				// 取得request中的所有文件名
				Iterator<String> iter = multiRequest.getFileNames();
				// 取得上传文件
				MultipartFile file = multiRequest.getFile(iter.next());
				
				if (file != null && file.getSize() >= 0) {
					if (file.getSize() > 32505856) {
						result.setStatus(SimpleController.ERROR);
						result.setMsg("上传文件过大!");
						return result;
					}
				// 取得当前上传文件的文件名称
				String myFileName = file.getOriginalFilename();
				// 获取文件后缀
				String filetype = myFileName.substring(myFileName.lastIndexOf(".") + 1);
				// 如果所属后缀为不可上传后缀，则返回
				if (!noStuffixSet.contains(filetype)) {
					result.setStatus(SimpleController.ERROR);
					result.setMsg("文件类型不支持!");
					return result;
				}
				String fileId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
				String folder = getFilePath(request);
				String path = folder+File.separator+fileId+"."+filetype;
				String smallName = fileId+"_small"+"."+filetype;
				String smallPath = folder+File.separator+smallName;
				File distFile = new File(path);
				if (!distFile.getParentFile().exists()){
					distFile.getParentFile().mkdirs();
				}		
					
					file.transferTo(distFile);
					smallPicture(distFile,smallPath,smallName);
					String str = path.substring(path.indexOf("upload")-1)+","+smallPath.substring(path.indexOf("upload")-1);
					result.setStatus(SimpleController.OK);
					result.addData("url", str);
				} else {
					result.setStatus(SimpleController.ERROR);
					result.setMsg("上传文件不能为空!");
					return result;
				}
			}
		} catch (Exception e) {
			result.setStatus(SimpleController.ERROR);
			result.setMsg("上传文件出现异常!");
			return result;
		}
		return result;
	}
	
	private String getFilePath(HttpServletRequest request){
		// 获取当前上传的年月日
		Calendar cal = Calendar.getInstance();
		// 当前年
		int year = cal.get(Calendar.YEAR);
		// 当前月
		int month = cal.get(Calendar.MONTH) + 1;
		// 当前日
		int day = cal.get(Calendar.DATE);
		//获取文件上传目录
		//获取项目目录
		String realPath = request.getSession().getServletContext().getRealPath("/");
		//获取项目的同级目录
		String root = new File(realPath).getParentFile().getAbsolutePath();
		//到配置文件中获取文件存放的目录
		String upload = config.getResourceUploadDir();
		String path = root+File.separator+upload+File.separator+"zone"+File.separator+year + File.separator + month + File.separator + day;
		return path;
	}
	
	//生成缩略图
	public void smallPicture(File sourceFile,String pathSmall,String uploadName) throws IOException{
		//新建缩略图文件
		File smallFile = new File(pathSmall);
		if (!smallFile.getParentFile().exists()){
			smallFile.getParentFile().mkdirs();
		}	
		//构造image图片
		Image src = javax.imageio.ImageIO.read(sourceFile);
		//表示该图像具有打包成整数的像素的8 位RGB颜色分量
		BufferedImage tag = new BufferedImage(150, 60, BufferedImage.TYPE_INT_BGR);
		//h绘制缩小后的图片
		tag.getGraphics().drawImage(src, 0, 0, 150, 60, null);
		//生成缩略图
		ImageIO.write(tag, "jpg", smallFile);
	}
*/
}

